#include <libps.h>
#include "MyPad.h"
#include "Line.h"

#define OT_LENGTH		2
#define PACKETMAX		2048
#define PACKETMAX2		(PACKETMAX*24) 
#define SCREEN_WIDTH	320
#define SCREEN_HEIGHT	256

GsOT		WorldOT[2];	
GsOT_TAG	OTTags[2][1<<OT_LENGTH];
static PACKET packetArea[2][PACKETMAX2];
volatile u_char *bb0, *bb1;

void main(void);
void InitGame(void);
static u_long ReadPad(void);


void main(void) {
	int activeBuff;
	u_long pad;
	GsGLINE lineST, lineXO;
	Line lST, lXO;
	
	lineST.attribute = lineXO.attribute = 0;
	lineST.x0 = 100; lineST.y0 = 100;
	lineST.x1 = 150; lineST.y1 = 150;
	lineXO.x0 = 150; lineXO.y0 = 100;
	lineXO.x1 = 100; lineXO.y1 = 150;
	SetRGB0(&lineST, 255, 96, 128);
	SetRGB1(&lineST, 0, 255, 0);
	SetRGB0(&lineXO, 0, 0, 255);
	SetRGB1(&lineXO, 255, 0, 0);
	
	
	InitGame();

	do {
		pad = ReadPad();
		activeBuff = GsGetActiveBuff();
		GsSetWorkBase((PACKET *)packetArea[activeBuff]);
		GsClearOt(0, 0, &WorldOT[activeBuff]);
		
		GsSortGLine(&lineST, &WorldOT[activeBuff], 0);
		GsSortGLine(&lineXO, &WorldOT[activeBuff], 1);
		
		if (pad & PADLdown) {
			if (pad & PADLX) lineXO.y0++;
			if (pad & PADLO) lineXO.y1++;
			if (pad & PADLS) lineST.y0++;
			if (pad & PADLT) lineST.y1++;
		}
		if (pad & PADLup) {
			if (pad & PADLX) lineXO.y0--;
			if (pad & PADLO) lineXO.y1--;
			if (pad & PADLS) lineST.y0--;
			if (pad & PADLT) lineST.y1--;
		}
		if (pad & PADLright) {
			if (pad & PADLX) lineXO.x0++;
			if (pad & PADLO) lineXO.x1++;
			if (pad & PADLS) lineST.x0++;
			if (pad & PADLT) lineST.x1++;
		}
		if (pad & PADLleft) {
			if (pad & PADLX) lineXO.x0--;
			if (pad & PADLO) lineXO.x1--;
			if (pad & PADLS) lineST.x0--;
			if (pad & PADLT) lineST.x1--;
		}
		lXO.sx = lineXO.x0;
		lXO.ex = lineXO.x1;
		lXO.sy = lineXO.y0;
		lXO.ey = lineXO.y1;
		lST.sx = lineST.x0;
		lST.ex = lineST.x1;
		lST.sy = lineST.y0;
		lST.ey = lineST.y1;
		
		DrawSync(0);
		VSync(0);
		GsSwapDispBuff();
		GsSortClear(0, 0, 0, &WorldOT[activeBuff]);
		GsDrawOt(&WorldOT[activeBuff]);
		FntPrint(0, "%d\n", LineCollision(lXO, lST));
		FntFlush(-1);
	} while ((pad != PADLstart+PADLselect) && (pad != PADRselect+PADRstart));
}




void InitGame(void) {
	int i;
	
	SetVideoMode(MODE_PAL);
	GsInitGraph(SCREEN_WIDTH, SCREEN_HEIGHT, GsINTER|GsOFSGPU, 1, 0);
	
	GsDefDispBuff(0, 0, 0, SCREEN_HEIGHT);
	GsDISPENV.screen.x = 7;
	GsDISPENV.screen.y = 16;
	GsDISPENV.screen.h = 255;
	
	GetPadBuf(&bb0, &bb1);
	
	for (i = 0; i < 2; i++) {
		WorldOT[i].length = OT_LENGTH;
		WorldOT[i].org = OTTags[i];
		GsClearOt(0,0,&WorldOT[i]);
	}
	
	FntLoad(960, 256);	
	FntOpen(20, 200, 320, 64, 0, 16);
}


u_long ReadPad(void) {
	return(~(*(bb0+3) | *(bb0+2) << 8 | *(bb1+3) << 16 | *(bb1+2) << 24));
}
